require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "android.net.Uri"
import "android.os.Environment"
import "android.graphics.pdf.PdfDocument"
import "java.io.*"

-- Tạo thư mục nếu chưa tồn tại
local function createDirectory(path)
    local dir = File(path)
    if not dir.exists() then
        dir.mkdirs()
    end
end

-- Danh sách định dạng hợp lệ
local supportedFormats = {".txt", ".docs", ".docx", ".pdf", ".rtf", ".odt", ".ott", ".doc", ".wps",
".sxw", ".pages", ".tex", ".log", ".md", ".mdx", ".rst", ".wri", ".adoc",
".asciidoc", ".nfo", ".me", ".1st", ".man", ".lit", ".srt", ".sub",
".texi", ".roff", ".troff", ".creole", ".markdown", ".dita", ".chm", ".ris", ".bib", ".bibtex",
".xps", ".eml", ".msg", ".xls", ".xlsx", ".xlsm", ".ods", ".fods", ".csv", ".tsv", ".numbers",
".dif", ".slk", ".ppt", ".pptx", ".odp", ".key", ".pot", ".potx",
".sxi", ".html", ".htm", ".xhtml", ".xml", ".xsl", ".json", ".yaml", ".yml", ".svg",
".webp", ".rss", ".py", ".pyc", ".pyo", ".pyw", ".ipynb", ".java", ".class", ".jar", ".jsp",
".c", ".cpp", ".cxx", ".cc", ".h", ".hpp", ".cs", ".js", ".mjs", ".cjs", ".jsx",
".ts", ".tsx", ".php", ".phtml", ".rb", ".erb", ".go", ".rs", ".swift",
".kt", ".kts", ".sh", ".bash", ".zsh", ".fish", ".bat", ".cmd", ".pl", ".pm",
".r", ".scala", ".sql", ".lua", ".hs", ".lhs", ".jl", ".dart", ".erl", ".hrl",
".ex", ".exs", ".m", ".mm", ".f", ".f90", ".f95", ".cbl", ".cob", ".vbs",
".vb", "Makefile", ".mk", "Dockerfile", ".coffee", ".clj", ".cljs", ".v",
".vhd", ".vhdl", ".rkt", ".adb", ".ads",
".phar", ".mjs", ".gem", ".dart_tool", ".asm", ".s",
".ini", ".cfg", ".conf", ".env", ".gitignore", ".htaccess", ".editorconfig",
".crt", ".pem", ".key", ".yaml", ".json5",
".epub", ".mobi", ".azw3", ".fb2", ".opf", ".azw", ".ibooks",
".dita", ".db", ".sqlite", ".properties", ".plist", ".toml", ".bib",
".accdb", ".mdb", ".out", ".dbf", ".jsonl",
".psd", ".ai", ".fig", ".sketch", ".xd"}

-- Kiểm tra định dạng tệp
local function isSupportedFormat(extension)
    for _, ext in ipairs(supportedFormats) do
        if ext:lower() == extension:lower() then return true end
    end
    return false
end

-- Lưu văn bản dưới dạng tệp văn bản
function saveAsTextFile(text, filePath)
    local success, err = pcall(function()
        local fos = FileOutputStream(filePath)
        local writer = OutputStreamWriter(fos)
        writer.write(text)
        writer.close()
        fos.close()
        service.asyncSpeak("tạo tệp văn bản thành công, tệp được lưu tại thư mục: " .. filePath)
        showOptionsAndExit(filePath)
    end)

    if not success then
        service.speak("Không thể lưu tệp văn bản: " .. tostring(err))
    end
end

-- Lấy phần mở rộng từ tên tệp hoặc Spinner
local function getFileExtension(fileName, selectedExtension)
    local inputExtension = fileName:match("%.([%w]+)$")
    if inputExtension and isSupportedFormat("." .. inputExtension) then
        return "." .. inputExtension
    else
        return selectedExtension
    end
end

-- Chuyển đổi văn bản sang định dạng tệp
function convertTextToFile(text, fileName, selectedExtension)
    local folderPath = Environment.getExternalStorageDirectory().getAbsolutePath() .. "/ghi chú/"
    createDirectory(folderPath)

    local extension = getFileExtension(fileName, selectedExtension)
    local fileNameWithoutExtension = fileName:gsub("%.[%w]+$", "")
    local filePath = folderPath .. fileNameWithoutExtension .. extension

    if extension == ".pdf" then
        saveAsPDF(text, filePath)
    elseif isSupportedFormat(extension) then
        saveAsTextFile(text, filePath)
    else
        service.speak("Định dạng không được hỗ trợ.")
    end
end

-- Lưu dưới dạng PDF
function saveAsPDF(text, filePath)
    local success, err = pcall(function()
        local pdfDoc = PdfDocument()
        local pageInfo = PdfDocument.PageInfo.Builder(595, 842, 1).create()
        local page = pdfDoc.startPage(pageInfo)
        local canvas = page.getCanvas()
        local paint = Paint()
        paint.setTextSize(12)
        paint.setColor(0xFF000000)

        local y = 25
        local lineHeight = 15
        local maxWidth = 575

        text = sanitizeText(text)
        for line in text:gmatch("[^\n]+") do
            if paint.measureText(line) > maxWidth then
                local words = {}
                for word in line:gmatch("%S+") do table.insert(words, word) end

                local currentLine = ""
                for _, word in ipairs(words) do
                    local tempLine = currentLine .. (currentLine == "" and "" or " ") .. word
                    if paint.measureText(tempLine) > maxWidth then
                        canvas.drawText(currentLine, 10, y, paint)
                        y = y + lineHeight
                        currentLine = word
                    else
                        currentLine = tempLine
                    end
                end
                canvas.drawText(currentLine, 10, y, paint)
                y = y + lineHeight
            else
                canvas.drawText(line, 10, y, paint)
                y = y + lineHeight
            end
        end

        pdfDoc.finishPage(page)

        local fos = FileOutputStream(filePath)
        pdfDoc.writeTo(fos)
        pdfDoc.close()
        fos.close()

        service.speak("tệp PDF đã được tạo tại: " .. filePath)
        showOptionsAndExit(filePath)
    end)

    if not success then
        service.speak("Lỗi khi tạo tệp PDF: " .. tostring(err))
    end
end

-- Chuẩn hóa văn bản
function sanitizeText(text)
    return text:gsub("\r\n", "\n"):gsub("\r", "\n")
end

-- Hiển thị tùy chọn và quay về màn hình chính
function showOptionsAndExit(filePath)
    local function closeAllDialogs()
        if dlg then dlg.dismiss() end
        if mainDlg then mainDlg.dismiss() end
    end

    local layout = {
        LinearLayout,
        orientation = "vertical",
        {
            Button,
            text = "Xóa",
            onClick = function()
                deleteFile(filePath)
                closeAllDialogs()
            end
        },
        {
            Button,
            text = "Chia sẻ",
            onClick = function()
                service.shareFile(filePath)
      service.postSpeak(400,"Chia sẻ tệp: ".. filePath)
                closeAllDialogs()
            end
        },
        {
            Button,
            text = "Thoát",
            onClick = function()
                closeAllDialogs()
service.postSpeak(500, "Tạm biệt")
            end
        }
    }

    dlg = LuaDialog(this)
    dlg.setTitle("Tuỳ chọn đường dẫn tệp:".. filePath)
    dlg.View = loadlayout(layout)
    dlg.show()
end

-- Xóa tệp
function deleteFile(filePath)
    local file = File(filePath)
    if file.exists() then
        file.delete()
        service.postSpeak(700,"Đã xóa tệp: ".. filePath.. "thành công.")
    else
        service.postSpeak(800, "Tệp không tồn tại.")
    end
end

-- Giao diện người dùng
function onClick()
    local layout = {
        ScrollView,
        layout_width = "fill",
        layout_height = "fill",
        {
            LinearLayout,
            orientation = "vertical",
            layout_width = "fill",
            layout_height = "fill",

            -- Ô nhập tên tệp
            {
                EditText,
                id = "fileName",
                hint = "Nhập tên tệp",
                layout_width = "match_parent",
                layout_marginTop = "16dp",
                layout_marginBottom = "8dp"
            },

{TextView,
text = "Chọn định dạng tệp:",
layout_width = "wrap_content",
layout_height = "wrap_content",
},
            -- Spinner chọn phần mở rộng tệp (di chuyển lên trên ô nhập nội dung)
            {
                Spinner,
                id = "fileExtensionSpinner",
                layout_width = "match_parent",
                layout_marginBottom = "8dp",
                adapter = ArrayAdapter(this, android.R.layout.simple_spinner_item, supportedFormats)
            },

            -- Ô nhập nội dung văn bản
            {
                EditText,
                id = "editText",
                hint = "Nhập văn bản",
                layout_width = "match_parent",
                layout_height = "0dp",
                layout_weight = 1,
                layout_marginBottom = "8dp"
            },

            -- Hàng chứa các nút
            {
                LinearLayout,
                orientation = "horizontal",
                layout_width = "match_parent",
                layout_height = "wrap_content",
                gravity = "center",

    {
        Button,
        text = "Xóa tất cả",
        layout_width = "0dp",  -- Đặt chiều rộng là 0dp
        layout_weight = 1,      -- Chia đều không gian
        onClick = function()
            editText.setText("")  -- Xóa nội dung trong ô nhập văn bản
            service.speak("Đã xóa văn bản")
        end
    },

    { Space, layout_width = "8dp" },

    {
        Button,
        text = "Dán văn bản từ bộ nhớ tạm",
        layout_width = "0dp",  -- Đặt chiều rộng là 0dp
        layout_weight = 1,      -- Chia đều không gian
        onClick = function()
            local clipboard = service.getSystemService(Context.CLIPBOARD_SERVICE)
            local clipData = clipboard.getPrimaryClip()
            if clipData ~= nil and clipData.getItemCount() > 0 then
                local pasteText = clipData.getItemAt(0).getText()
                if pasteText then
                    editText.setText(pasteText)
                    service.speak("Đã dán văn bản từ bộ nhớ tạm")
                else
                    service.speak("Không có văn bản trong bộ nhớ tạm")
                end
            else
                service.speak("Không có gì trong bộ nhớ tạm")
            end
        end
    },

    { Space, layout_width = "8dp" },

    {
        Button,
        text = "Dán văn bản từ mục yêu thích",
        layout_width = "0dp",  -- Đặt chiều rộng là 0dp
        layout_weight = 1,      -- Chia đều không gian
        onClick = function()
            local favorites = service.getFavoritesList()
            if #favorites > 0 then
                local favoriteText = favorites[0]  -- Lấy mục yêu thích đầu tiên
                editText.setText(favoriteText)  -- Dán nội dung vào ô nhập văn bản
                service.speak("Đã dán văn bản từ mục yêu thích")
            else
                service.speak("Không có văn bản trong mục yêu thích")
            end
        end
    },

{ Space, layout_width = "8dp" },

                {
                    Button,
                    text = "Lưu",
                    layout_weight = 1,
                    layout_width = "0dp",
                    onClick = function()
                        local text = editText.getText().toString()
                        local fileName = fileName.getText().toString()
                        local selectedExtension = fileExtensionSpinner.getSelectedItem()

                        if text == "" then
                            service.speak("Vui lòng nhập văn bản.")
                        elseif fileName == "" then
                            service.speak("Vui lòng nhập tên tệp.")
                        else
                            convertTextToFile(text, fileName, selectedExtension)
                        end
                    end
                },

                { Space, layout_width = "8dp" },

                {
                    Button,
                    text = "Thoát",
                    layout_weight = 1,
                    layout_width = "0dp",
                    onClick = function()
                        if mainDlg then mainDlg.dismiss() end
                    end
                }
            }
        }
    }

    mainDlg = LuaDialog(this)
    mainDlg.setTitle("Tạo tệp văn bản")
    mainDlg.View = loadlayout(layout)
    mainDlg.show()
end

-- Khởi chạy ứng dụng
onClick()
